// **********************************************************************
// SCIDYNE   649 School Street   Pembroke, MA 02359-3632 USA
// Tel: (781) 293-3059   Fax: (781) 293-4034   URL: www.scidyne.com
// **********************************************************************
// Project:  GPIO-104 Demonstration Program
// File:     GPIO_EX1.CPP
// Revision: 1.00
// Date:     03-05-99
// Author:   Mark W. Durgin
// *************************************************************************
// This C program demonstrates the basic operation of the SCIDYNE GPIO-104
// data acquistion module (Model 100-7602). For clarity and ease in
// understanding the program has been written with simplicity taking
// precedence over robustness. It is supplied without any implied or
// expressed warranties or claims. The user accepts all consequences
// resulting from its use and has the sole responsibility in determining its
// suitability for each application.
//
// The program operates by first initializing important system variables and
// then runs in a continous loop until stopped by pressing the 'X' key.
// The analog inputs are digitized and displayed as raw numbers. The analog
// output values are also displayed and can be indiviually incremented or
// decremented by pressing certain keys.
//
// The eight analog inputs are setup to measure voltages in the range of
// +/- 10Vdc using internally timed acquisition.  The setup parameters
// and resulting values are stored in an array of structures where each
// member structure corresponds to a separate analog input channel. For
// simplicity the DAS chip operations are polled but the code easily changed
// to use interrupts if desired.
//
// Each of the analog outputs are first pre-loaded with their corresponding
// 12-bit value and then updated simultaneously. It should be noted that
// although this program continuously updates the analog outputs this is
// not necessary since they will hold the last value written.
//
// The 24 digital line are initialized as outputs.  During operation PORTA
// will output the binary LSB of analog out#0 while PORTB outputs the MSB.
// PORTC is not used by this program.
// *************************************************************************
// History:
//   12-08-98  R1.00  Original Code
// *************************************************************************

// --- Program includes
#include <stdio.h>
#include <dos.h>
#include <conio.h>

// --- Define program constants
#define	TRUE								1									// Logic TRUE state
#define FALSE								!TRUE							// Logic FALSE state
#define	BASE_ADDR 					0x300							// Define where in IO map GPIO is located

#define INTR_STATUS 				BASE_ADDR+0  			// Intrrupt Status register
#define	AO_UPDATE						BASE_ADDR+1       // Write to simultaneously update all analog outputs

#define DAS_CTRL 						BASE_ADDR+2       // Das control register
#define	DAS_LB							BASE_ADDR+2       // 12-bit Result LB
#define	DAS_HB							BASE_ADDR+3       // 12-bit result HB

#define AOCH0_LB						BASE_ADDR+4       // 12-bit analog out #0 LB
#define AOCH0_HB						BASE_ADDR+5       // 12-bit analog out #0 HB
#define AOCH1_LB						BASE_ADDR+6       // 12-bit analog out #1 LB
#define AOCH1_HB						BASE_ADDR+7       // 12-bit analog out #1 HB
#define AOCH2_LB						BASE_ADDR+8       // 12-bit analog out #2 LB
#define AOCH2_HB						BASE_ADDR+9       // 12-bit analog out #2 HB
#define AOCH3_LB						BASE_ADDR+10      // 12-bit analog out #3 LB
#define AOCH3_HB						BASE_ADDR+11      // 12-bit analog out #3 HB

#define PORTA								BASE_ADDR+12      // 82C55 PORT-A
#define PORTB								BASE_ADDR+13      // 82C55 PORT-B
#define PORTC								BASE_ADDR+14      // 82C55 PORT-C
#define DIO_CTRL						BASE_ADDR+15      // 82C55 Control Register

// --- MAX197 Control register bit positions and single bit masks
#define CTRL_BYTE_BIP_BIT 		3      // Conversion: 0 = Unipolar, 1 = Bipolar
#define CTRL_BYTE_RNG_BIT 		4      // Full Scale: 0 = 5, 1 = 10V
#define CTRL_BYTE_ACQMOD_BIT	5      // Acquistion: 0 = Internal, 1 = External
#define CTRL_BYTE_PD0_BIT 		6      // Clock and Power Down
#define CTRL_BYTE_PD1_BIT 		7      // Clock and Power Down

#define CTRL_BYTE_BIP_MASK 		(1 << CTRL_BYTE_BIP_BIT)
#define CTRL_BYTE_RNG_MASK 		(1 << CTRL_BYTE_RNG_BIT)
#define CTRL_BYTE_ACQMOD_MASK	(1 << CTRL_BYTE_ACQMOD_BIT)
#define CTRL_BYTE_PD0_MASK		(1 << CTRL_BYTE_PD0_BIT)
#define CTRL_BYTE_PD1_MASK		(1 << CTRL_BYTE_PD1_BIT)

// --- Interrupt Status register bit positions and single bit masks
#define INTR_STATUS_DAS_BIT 	0   		// DAS Conversion Status:  1 = New value ready
#define INTR_STATUS_EXT_BIT 	1       // External Interrupt Status:  1 = External Interrupt

#define INTR_STATUS_DAS_MASK  (1 << INTR_STATUS_DAS_BIT)
#define INTR_STATUS_EXT_MASK  (1 << INTR_STATUS_EXT_BIT)

#define DIO_MODE							0x80 		// Defines mode of 82C55 ports
#define AO_INC_DEC_STEP				10 			// Defines step change of analog outputs when key is pressed

// --- Declare function prototypes
void program_initz(void);     				// Performs one-time program setup

// --- Declare global variables and strucures
union  bytes_or_word{    							// Variables created using this union
	unsigned char 	byte[2];    				// can be accessed as either 2 bytes or
	int 						word;       				// as one word. Note: Byte[0] = LSB
};

struct {  // Holds setups and results for corresponding analog inputs
	union  		bytes_or_word 	result;    		// Holds analog input result
	unsigned 	char						das_ctrl;   	// Holds the DAS channel control parameters
} das_channel[8];

struct {  // Holds data values for corresponding analog outputs
	union  		bytes_or_word 	value;    		// Holds analog output value
} dac_channel[4];

// --- This is the main loop
void main(void)
{
	//--- Declare local variables
	int x;				// General purpose variable

	//--- Initz the program
	program_initz();

  //--- This is the main loop
  while(TRUE)
	{
		//--- Digitize and store each DAS channel using parameters from structure
		for(x=0;x<8;x++)
		{
			outp(DAS_CTRL,das_channel[x].das_ctrl);   						// Start a convesion on the designated channel

			while(!(inp(INTR_STATUS) & INTR_STATUS_DAS_MASK));  	// Poll DAS until conversion is complete

			das_channel[x].result.byte[0] = inp(DAS_LB); 					// Read and store the DAS result LB
			das_channel[x].result.byte[1] = inp(DAS_HB);   				// Read and store the DAS result HB
		}


		//--- Check if user is making changes to analog outputs or
		//    requesting the program to end
		if(kbhit())
		{
			switch (getch())  // Read the keystroke
			{
				case '1':  // Dec AOCH0
				{
					dac_channel[0].value.word -= AO_INC_DEC_STEP;
					if (dac_channel[0].value.word < 0)
						dac_channel[0].value.word = 0;
					break;
				}

				case '2': // Inc AOCH0
				{
					dac_channel[0].value.word += AO_INC_DEC_STEP;
					if (dac_channel[0].value.word > 4095)
						dac_channel[0].value.word = 4095;
					break;
				}

				case '3':  // Dec AOCH1
				{
					dac_channel[1].value.word -= AO_INC_DEC_STEP;
					if (dac_channel[1].value.word < 0)
						dac_channel[1].value.word = 0;
					break;
				}

				case '4': // Inc AOCH1
				{
					dac_channel[1].value.word += AO_INC_DEC_STEP;
					if (dac_channel[1].value.word > 4095)
						dac_channel[1].value.word = 4095;
					break;
				}

				case '5':  // Dec AOCH2
				{
					dac_channel[2].value.word -= AO_INC_DEC_STEP;
					if (dac_channel[2].value.word < 0)
						dac_channel[2].value.word = 0;
					break;
				}

				case '6': // Inc AOCH2
				{
					dac_channel[2].value.word += AO_INC_DEC_STEP;
					if (dac_channel[2].value.word > 4095)
						dac_channel[2].value.word = 4095;
					break;
				}

				case '7':  // Dec AOCH3
				{
					dac_channel[3].value.word -= AO_INC_DEC_STEP;
					if (dac_channel[3].value.word < 0)
						dac_channel[3].value.word = 0;
					break;
				}

				case '8': // Inc AOCH3
				{
					dac_channel[3].value.word += AO_INC_DEC_STEP;
					if (dac_channel[3].value.word > 4095)
						dac_channel[3].value.word = 4095;
					break;
				}

				case 'x':
				case 'X':  // Quit the program
				{
					return;
					break;
				}

				default:
					break;
			}
		}

		//--- Refresh the display
		gotoxy (1,7); // Locate the cursor

		//--- Show the analog input results
		printf("--- Analog inputs ---\n");
		for(x=0; x<8; x++)
		{
			printf("DAS CH-%d: %d \n",x,das_channel[x].result.word);
		}

		//--- Display and PRELOAD the analog output values
		printf("\n--- Analog Outputs ---\n");

		//--- Preload analog output channel #0
		printf("DAC CH-0 %d (dec= 1  inc= 2) \n", dac_channel[0].value.word);
		outp(AOCH0_LB, (dac_channel[0].value.byte[0]));
		outp(AOCH0_HB, (dac_channel[0].value.byte[1]));

		//--- Preload analog output channel #1
		printf("DAC CH-1 %d (dec= 3  inc= 4) \n", dac_channel[1].value.word);
		outp(AOCH1_LB, (dac_channel[1].value.byte[0]));
		outp(AOCH1_HB, (dac_channel[1].value.byte[1]));

		//--- Preload analog output channel #2
		printf("DAC CH-2 %d (dec= 5  inc= 6) \n", dac_channel[2].value.word);
		outp(AOCH2_LB, (dac_channel[2].value.byte[0]));
		outp(AOCH2_HB, (dac_channel[2].value.byte[1]));

		//--- Preload analog output channel #3
		printf("DAC CH-3 %d (dec= 7  inc= 8) \n", dac_channel[3].value.word);
		outp(AOCH3_LB, (dac_channel[3].value.byte[0]));
		outp(AOCH3_HB, (dac_channel[3].value.byte[1]));

		//--- Update all the analog outputs at once
		outp(AO_UPDATE,0x00);  	// Any value written to this location will work

		//--- Show digital IO port operation by writing the 12-bit binary value
		// of AOCH0 on PORTA (LSB) and PORTB (MSB)
		outp(PORTA, dac_channel[0].value.byte[0]);   // Show LSB
		outp(PORTB, dac_channel[0].value.byte[1]);   // Show MSB

	}
}


//--- Intiz the program
void program_initz(void)
{
	//--- Clear the display and show a banner
	clrscr();
	printf ("------------------------------\n");
	printf ("         SCIDYNE\n");
	printf ("GPIO-104 Demonstration Program\n");
	printf ("  gpio_ex1.cpp     V1.00\n");
	printf ("------------------------------\n");

	//--- Setup and initz the digital IO
	outp(DIO_CTRL,0x80);   	// Make all ports of 82C55 outputs
	outp(PORTA,0);  				// Force all bits of PORTA to 0 (low)
	outp(PORTB,0);  				// Force all bits of PORTB to 0 (low)
	outp(PORTC,0);  				// Force all bits of PORTC to 0 (low)

	//--- Initz DAS channel 0 operating parameters
	das_channel[0].result.word = 0;  // Initz DAS result to zero
	das_channel[0].das_ctrl =     		// Set DAS channel #0 parameters
		!CTRL_BYTE_PD1_MASK 		+  		// Bit7: PD1 MUST be 0 for external clock
		!CTRL_BYTE_PD0_MASK			+  		// Bit6: PD0 MUST be 0 for external clock
		!CTRL_BYTE_ACQMOD_MASK 	+  		// Bit5: ACQMOD = 0 	(0 = Internal, 1=External)
		 CTRL_BYTE_RNG_MASK 		+  		// Bit4: RNG = 1  		(0=5V, 1=10V Full Scale)
		 CTRL_BYTE_BIP_MASK     +  		// Bit3: BIP = 1  		(0 = Unipolar, 1=Bipolar)
		 0;                        		// Bit2-0: Channel address = 0

	//--- Initz DAS channel 1 operating parameters
	das_channel[1].result.word = 0;  // Initz DAS result to zero
	das_channel[1].das_ctrl =     		// Set DAS channel #1 parameters
		!CTRL_BYTE_PD1_MASK 		+  		// Bit7: PD1 MUST be 0 for external clock
		!CTRL_BYTE_PD0_MASK			+  		// Bit6: PD0 MUST be 0 for external clock
		!CTRL_BYTE_ACQMOD_MASK 	+  		// Bit5: ACQMOD = 0 	(0 = Internal, 1=External)
		 CTRL_BYTE_RNG_MASK 		+  		// Bit4: RNG = 1  		(0=5V, 1=10V Full Scale)
		 CTRL_BYTE_BIP_MASK     +  		// Bit3: BIP = 1  		(0 = Unipolar, 1=Bipolar)
		 1;                        		// Bit2-0: Channel address = 1

	//--- Initz DAS channel 2 operating parameters
	das_channel[2].result.word = 0;  // Initz DAS result to zero
	das_channel[2].das_ctrl =     		// Set DAS channel #2 parameters
		!CTRL_BYTE_PD1_MASK 		+  		// Bit7: PD1 MUST be 0 for external clock
		!CTRL_BYTE_PD0_MASK			+  		// Bit6: PD0 MUST be 0 for external clock
		!CTRL_BYTE_ACQMOD_MASK 	+  		// Bit5: ACQMOD = 0 	(0 = Internal, 1=External)
		 CTRL_BYTE_RNG_MASK 		+  		// Bit4: RNG = 1  		(0=5V, 1=10V Full Scale)
		 CTRL_BYTE_BIP_MASK     +  		// Bit3: BIP = 1  		(0 = Unipolar, 1=Bipolar)
		 2;                        		// Bit2-0: Channel address = 2

	//--- Initz DAS channel 3 operating parameters
	das_channel[3].result.word = 0;  // Initz DAS result to zero
	das_channel[3].das_ctrl =     		// Set DAS channel #3 parameters
		!CTRL_BYTE_PD1_MASK 		+  		// Bit7: PD1 MUST be 0 for external clock
		!CTRL_BYTE_PD0_MASK			+  		// Bit6: PD0 MUST be 0 for external clock
		!CTRL_BYTE_ACQMOD_MASK 	+  		// Bit5: ACQMOD = 0 	(0 = Internal, 1=External)
		 CTRL_BYTE_RNG_MASK 		+  		// Bit4: RNG = 1  		(0=5V, 1=10V Full Scale)
		 CTRL_BYTE_BIP_MASK     +  		// Bit3: BIP = 1  		(0 = Unipolar, 1=Bipolar)
		 3;                        		// Bit2-0: Channel address = 3

	//--- Initz DAS channel 4 operating parameters
	das_channel[4].result.word = 0;  // Initz DAS result to zero
	das_channel[4].das_ctrl =     		// Set DAS channel #4 parameters
		!CTRL_BYTE_PD1_MASK 		+  		// Bit7: PD1 MUST be 0 for external clock
		!CTRL_BYTE_PD0_MASK			+  		// Bit6: PD0 MUST be 0 for external clock
		!CTRL_BYTE_ACQMOD_MASK 	+  		// Bit5: ACQMOD = 0 	(0 = Internal, 1=External)
		 CTRL_BYTE_RNG_MASK 		+  		// Bit4: RNG = 1  		(0=5V, 1=10V Full Scale)
		 CTRL_BYTE_BIP_MASK     +  		// Bit3: BIP = 1  		(0 = Unipolar, 1=Bipolar)
		 4;                        		// Bit2-0: Channel address = 4

	//--- Initz DAS channel 5 operating parameters
	das_channel[5].result.word = 0;  // Initz DAS result to zero
	das_channel[5].das_ctrl =     		// Set DAS channel #5 parameters
		!CTRL_BYTE_PD1_MASK 		+  		// Bit7: PD1 MUST be 0 for external clock
		!CTRL_BYTE_PD0_MASK			+  		// Bit6: PD0 MUST be 0 for external clock
		!CTRL_BYTE_ACQMOD_MASK 	+  		// Bit5: ACQMOD = 0 	(0 = Internal, 1=External)
		 CTRL_BYTE_RNG_MASK 		+  		// Bit4: RNG = 1  		(0=5V, 1=10V Full Scale)
		 CTRL_BYTE_BIP_MASK     +  		// Bit3: BIP = 1  		(0 = Unipolar, 1=Bipolar)
		 5;                        		// Bit2-0: Channel address = 5

	//--- Initz DAS channel 6 operating parameters
	das_channel[6].result.word = 0;  // Initz DAS result to zero
	das_channel[6].das_ctrl =     		// Set DAS channel #6 parameters
		!CTRL_BYTE_PD1_MASK 		+  		// Bit7: PD1 MUST be 0 for external clock
		!CTRL_BYTE_PD0_MASK			+  		// Bit6: PD0 MUST be 0 for external clock
		!CTRL_BYTE_ACQMOD_MASK 	+  		// Bit5: ACQMOD = 0 	(0 = Internal, 1=External)
		 CTRL_BYTE_RNG_MASK 		+  		// Bit4: RNG = 1  		(0=5V, 1=10V Full Scale)
		 CTRL_BYTE_BIP_MASK     +  		// Bit3: BIP = 1  		(0 = Unipolar, 1=Bipolar)
		 6;                        		// Bit2-0: Channel address = 6

	//--- Initz DAS channel 7 operating parameters
	das_channel[7].result.word = 0;  // Initz DAS result to zero
	das_channel[7].das_ctrl =     		// Set DAS channel #7 parameters
		!CTRL_BYTE_PD1_MASK 		+  		// Bit7: PD1 MUST be 0 for external clock
		!CTRL_BYTE_PD0_MASK			+  		// Bit6: PD0 MUST be 0 for external clock
		!CTRL_BYTE_ACQMOD_MASK 	+  		// Bit5: ACQMOD = 0 	(0 = Internal, 1=External)
		 CTRL_BYTE_RNG_MASK 		+  		// Bit4: RNG = 1  		(0=5V, 1=10V Full Scale)
		 CTRL_BYTE_BIP_MASK     +  		// Bit3: BIP = 1  		(0 = Unipolar, 1=Bipolar)
		 7;                        		// Bit2-0: Channel address = 7

	//--- Initz DAC channels 0-3
	dac_channel[0].value.word = 2047;
	dac_channel[1].value.word = 2047;
	dac_channel[2].value.word = 2047;
	dac_channel[3].value.word = 2047;

}

